#include "Agent.h"
#include <stdio.h>

/**
 * Konstruktor Agent() vytvori agenta na serveri.
 * @param run - Premenna urcujuca, ci agent ma este bezat na serveri.
 * @param protocol - Protokol, cez ktory je mozne komunikovat so serverom.
 * Ak <i>protocol</i> je:
 * <ul>
 * 	<li> 6, tak agent bude komunikovat so serverom prostrednictvom protokolu TCP.</li>
 * 	<li> 17, tak agent bude komunikovat so serverom prostrednictvom protokolu UDP.</li>
 * </ul>
 * @param port - Port v "network byte order", na ktorom je mozne komunikovat so serverom.
 * @param address - Adresa servera v "network byte order".
 * @param team - Tim agenta.
 * @param id - Cislo agenta.
 * @param playGroundLength - Dlzka ihriska.
 * @param playGroundWidth - Sirka ihriska.
 * @param goalWidth - Sirka brany.
 * @param goalHeight - Vyska brany.
 */

Agent::Agent(bool *run, unsigned char protocol, unsigned short port, unsigned long address, char *team, int id, float playGroundLength, float playGroundWidth, float goalWidth, float goalHeight)
{
	this->run = run;
	response = new Response(team, id, playGroundLength, playGroundWidth, goalWidth, goalHeight);
	communication = new Communication(protocol);
	char sendingData[1025];
	char *sendingMessage = sendingData + 4;
#ifdef WIN32
	int sendingMessageSize = sizeof(sendingData) - 4;
#endif
	char receivingData[4097];
	if (!communication->setServer(port, address))
	{
		*run = false;
	}
	if (*run)
	{
#ifdef WIN32
		sprintf_s(sendingMessage, sendingMessageSize, "(scene rsg/agent/nao/nao.rsg)");
#else
		sprintf(sendingMessage, "(scene rsg/agent/nao/nao.rsg)");
#endif
	}
	if (*run && !communication->send(sendingMessage))
	{
		*run = false;
	}
	while (*run && !communication->select(1, 0))
	{
	}
	while (*run && communication->select(0, 0))
	{
		if (!communication->receive(receivingData + 4, sizeof(receivingData) - 4))
		{
			*run = false;
		}
	}
	if (*run)
	{
#ifdef WIN32
		sprintf_s(sendingMessage, sendingMessageSize, "(init (unum %i)(teamname %s))", id, team);
#else
		sprintf(sendingMessage, "(init (unum %i)(teamname %s))", id, team);
#endif
	}
	if (*run && !communication->send(sendingMessage))
	{
		*run = false;
	}
}

/**
 * Metoda start() umoznuje agentovi zacat hrat na serveri.
 */

void Agent::start()
{
	char sendingData[1025];
	char *sendingMessage = sendingData + 4;
	int sendingMessageSize = sizeof(sendingData) - 4;
	char receivingData[4097];
	char *receivingMessage = receivingData + 4;
	int receivingMessageSize = sizeof(receivingData) - 4;
	while (*run)
	{
		while (*run && !communication->select(1, 0))
		{
		}
		while (*run && communication->select(0, 0))
		{
			if (!communication->receive(receivingMessage, receivingMessageSize))
			{
				*run = false;
			}
		}
		if (*run)
		{
			response->nextStep(receivingMessage, sendingMessage, sendingMessageSize);
		}
		if (*run && !communication->send(sendingMessage))
		{
			*run = false;
		}
	}
}

/**
 * Destruktor ~Agent() zrusi agenta zo servera.
 */

Agent::~Agent()
{
	delete communication;
	delete response;
}
