#include "Behaviour.h"

/**
 * Konstruktor Behaviour() vytvori podtriedy, ktore budu potrebne na spravanie sa podla situacie vzniknutej na ihrisku.
 * @param team - Tim agenta.
 * @param id - Cislo agenta.
 * @param playGroundLength - Dlzka ihriska.
 * @param playGroundWidth - Sirka ihriska.
 * @param goalWidth - Sirka brany.
 * @param goalHeight - Vyska brany.
 */

Behaviour::Behaviour(char *team, int id, float playGroundLength, float playGroundWidth, float goalWidth, float goalHeight)
{
	worldModel = new WorldModel(team, id, playGroundLength, playGroundWidth, goalWidth, goalHeight);
	teamBehaviour = new TeamBehaviour();
	individualBehaviour = new IndividualBehaviour();
}

/**
 * Metoda nextStep() pomocou vytvorenych podtried umozni spravat sa podla situacie vzniknutej na ihrisku.
 * @param perceptorSet - Ukazovatel na sadu perceptorov, ktora popisuje situaciu vzniknutu na ihrisku.
 * @param effectorSet - Ukazovatel na sadu efektorov, ktorou sa popisuje spravanie sa podla situacie vzniknutej na ihrisku.
 */

void Behaviour::nextStep(PerceptorSet *perceptorSet, EffectorSet *effectorSet)
{
	worldModel->transform(perceptorSet);
	teamBehaviour->nextStep(worldModel);
	individualBehaviour->nextStep(worldModel);
	worldModel->reverseTransform(effectorSet);
}

/**
 * Destruktor ~Behaviour() zrusi podtriedy, ktore boli potrebne na spravanie sa podla situacie vzniknutej na ihrisku.
 */

Behaviour::~Behaviour()
{
	delete individualBehaviour;
	delete teamBehaviour;
	delete worldModel;
}
