#include "EffectorSet.h"

/**
 * Konstruktor EffectorSet() vytvori podtriedy, ktore budu potrebne na zlozenie posielanej spravy, ktora bude obsahovat efektory.
 */

EffectorSet::EffectorSet()
{
	beamCount = 0;
	beam = new Beam[1]();
	hingeJointCount = 0;
	hingeJoint = new HingeJoint[100]();
	sayCount = 0;
	say = new Say[1]();
}

/**
 * Metoda setBeamCount() nastavi pocet beam efektorov, ktore budu obsiahnute v posielanej sprave.
 * @param beamCount - Pocet beam efektorov.
 */

void EffectorSet::setBeamCount(int beamCount)
{
	this->beamCount = beamCount;
}

/**
 * Metoda getBeam() vrati ukazovatel na pozadovany beam efektor.
 * @param index - Cislo pozadovaneho beam efektoru.
 * @return Ukazovatel na pozadovany beam efektor.
 */

Beam *EffectorSet::getBeam(int index)
{
	return &beam[index];
}

/**
 * Metoda setHingeJointCount() nastavi pocet hingejoint efektorov, ktore budu obsiahnute v posielanej sprave.
 * @param hingeJointCount - Pocet hingejoint efektorov.
 */

void EffectorSet::setHingeJointCount(int hingeJointCount)
{
	this->hingeJointCount = hingeJointCount;
}

/**
 * Metoda getHingeJoint() vrati ukazovatel na pozadovany hingejoint efektor.
 * @param index - Cislo pozadovaneho hingejoint efektoru.
 * @return Ukazovatel na pozadovany hingejoint efektor.
 */

HingeJoint *EffectorSet::getHingeJoint(int index)
{
	return &hingeJoint[index];
}

/**
 * Metoda setSayCount() nastavi pocet say efektorov, ktore budu obsiahnute v posielanej sprave.
 * @param sayCount - Pocet say efektorov.
 */

void EffectorSet::setSayCount(int sayCount)
{
	this->sayCount = sayCount;
}

/**
 * Metoda getSay() vrati ukazovatel na pozadovany say efektor.
 * @param index - Cislo pozadovaneho say efektoru.
 * @return Ukazovatel na pozadovany say efektor.
 */

Say *EffectorSet::getSay(int index)
{
	return &say[index];
}

/**
 * Metoda compose() zlozi z prvkov podtried posielanu spravu, ktora bude obsahovat efektory.
 * @param sendingMessage - Vystupna posielana sprava, ktora bude obsahovat efektory.
 * @param sendingMessageSize - Maximalna dlzka posielanej spravy, ktora bude obsahovat efektory.
 */

void EffectorSet::compose(char *sendingMessage, int sendingMessageSize)
{
	int i = 0;
	for (int j = 0; j < beamCount; j ++)
	{
		i += beam[j].compose(sendingMessage + i, sendingMessageSize - i);
	}
	beamCount = 0;
	for (int j = 0; j < hingeJointCount; j ++)
	{
		i += hingeJoint[j].compose(sendingMessage + i, sendingMessageSize - i);
	}
	hingeJointCount = 0;
	for (int j = 0; j < sayCount; j ++)
	{
		i += say[j].compose(sendingMessage + i, sendingMessageSize - i);
	}
	sayCount = 0;
	sendingMessage[i] = '\0';
}

/**
 * Destruktor ~EffectorSet() zrusi podtriedy, ktore boli potrebne na zlozenie posielanej spravy, ktora obsahovala efektory.
 */

EffectorSet::~EffectorSet()
{
	delete[] say;
	delete[] hingeJoint;
	delete[] beam;
}
