#include "PerceptorSet.h"
#include <iostream>
#include <string.h>

using namespace std;

/**
 * Konstruktor PerceptorSet() vytvori podtriedy, ktore budu potrebne na rozlozenie prijatej spravy, ktora obsahuje perceptory.
 */

PerceptorSet::PerceptorSet()
{
	hjCount = 0;
	hj = new HJ[100]();
	frpCount = 0;
	frp = new FRP[100]();
	gyrCount = 0;
	gyr = new GYR[100]();
	time = new Time();
	gs = new GS();
	seeCount = 0;
	see = new See[1]();
	hearCount = 0;
	hear = new Hear[22]();
}

/**
 * Metoda decompose() rozlozi na prvky podtried prijatu spravu, ktora obsahuje perceptory.
 * @param receivingMessage - Prijata sprava, ktora obsahuje perceptory.
 */

void PerceptorSet::decompose(char *receivingMessage)
{
	hjCount = 0;
	frpCount = 0;
	gyrCount = 0;
	seeCount = 0;
	hearCount = 0;
	int i = 0;
	while (receivingMessage[i] != '\0')
	{
		if (receivingMessage[i] == '(')
		{
			char *receivingMessage2 = receivingMessage + i;
			char *receivingMessage3 = receivingMessage2 + 1;
			if (strncmp(receivingMessage3, "HJ ", 3) == 0)
			{
				i += hj[hjCount].decompose(receivingMessage2);
				hjCount ++;
			}
			else if (strncmp(receivingMessage3, "FRP ", 4) == 0)
			{
				i += frp[frpCount].decompose(receivingMessage2);
				frpCount ++;
			}
			else if (strncmp(receivingMessage3, "GYR ", 4) == 0)
			{
				i += gyr[gyrCount].decompose(receivingMessage2);
				gyrCount ++;
			}
			else if (strncmp(receivingMessage3, "time ", 5) == 0)
			{
				i += time->decompose(receivingMessage2);
			}
			else if (strncmp(receivingMessage3, "GS ", 3) == 0)
			{
				i += gs->decompose(receivingMessage2);
			}
			else if (strncmp(receivingMessage3, "See ", 4) == 0)
			{
				i += see[seeCount].decompose(receivingMessage2);
				seeCount ++;
			}
			else if (strncmp(receivingMessage3, "hear ", 5) == 0)
			{
				i += hear[hearCount].decompose(receivingMessage2);
				hearCount ++;
			}
			else
			{
				int j = 1;
				int k = 1;
				do
				{
					switch (receivingMessage2[j])
					{
						case '(':
							k ++;
							break;
						case ')':
							k --;
							break;
					}
					j ++;
				}
				while (k > 0);
				i += j;
				char *perceptor = new char[j + 1];
				memcpy(perceptor, receivingMessage2, j);
				perceptor[j] = '\0';
				cerr << "Agent nepozna perceptor: \"" << perceptor << "\"." << endl;
				delete[] perceptor;
			}
		}
		else
		{
			i ++;
		}
	}
}

/**
 * Metoda getHJCount() vrati pocet HJ_perceptorov, ktore boli obsiahnute v prijatej sprave.
 * @return Pocet HJ_perceptorov.
 */

int PerceptorSet::getHJCount()
{
	return hjCount;
}

/**
 * Metoda getHJ() vrati ukazovatel na pozadovany HJ_perceptor.
 * @param index - Cislo pozadovaneho HJ_perceptoru.
 * @return Ukazovatel na pozadovany HJ_perceptor.
 */

HJ *PerceptorSet::getHJ(int index)
{
	return &hj[index];
}

/**
 * Metoda getFRPCount() vrati pocet FRP_perceptorov, ktore boli obsiahnute v prijatej sprave.
 * @return Pocet FRP_perceptorov.
 */

int PerceptorSet::getFRPCount()
{
	return frpCount;
}

/**
 * Metoda getFRP() vrati ukazovatel na pozadovany FRP_perceptor.
 * @param index - Cislo pozadovaneho FRP_perceptoru.
 * @return Ukazovatel na pozadovany FRP_perceptor.
 */

FRP *PerceptorSet::getFRP(int index)
{
	return &frp[index];
}

/**
 * Metoda getGYRCount() vrati pocet GYR_perceptorov, ktore boli obsiahnute v prijatej sprave.
 * @return Pocet GYR_perceptorov.
 */

int PerceptorSet::getGYRCount()
{
	return gyrCount;
}

/**
 * Metoda geGYR() vrati ukazovatel na pozadovany GYR_perceptor.
 * @param index - Cislo pozadovaneho GYR_perceptoru.
 * @return Ukazovatel na pozadovany GYR_perceptor.
 */

GYR *PerceptorSet::getGYR(int index)
{
	return &gyr[index];
}

/**
 * Metoda getTime() vrati ukazovatel na time perceptor.
 * @return Ukazovatel na time perceptor.
 */

Time *PerceptorSet::getTime()
{
	return time;
}

/**
 * Metoda getGS() vrati ukazovatel na GS_perceptor.
 * @return Ukazovatel na GS_perceptor.
 */

GS *PerceptorSet::getGS()
{
	return gs;
}

/**
 * Metoda getSeeCount() vrati pocet See_perceptorov, ktore boli obsiahnute v prijatej sprave.
 * @return Pocet See_perceptorov.
 */

int PerceptorSet::getSeeCount()
{
	return seeCount;
}

/**
 * Metoda getSee() vrati ukazovatel na pozadovany See_perceptor.
 * @param index - Cislo pozadovaneho See_perceptoru.
 * @return Ukazovatel na pozadovany See_perceptor.
 */

See *PerceptorSet::getSee(int index)
{
	return &see[index];
}

/**
 * Metoda getHearCount() vrati pocet hear perceptorov, ktore boli obsiahnute v prijatej sprave.
 * @return Pocet hear perceptorov.
 */

int PerceptorSet::getHearCount()
{
	return hearCount;
}

/**
 * Metoda getHear() vrati ukazovatel na pozadovany hear perceptor.
 * @param index - Cislo pozadovaneho hear perceptoru.
 * @return Ukazovatel na pozadovany hear perceptor.
 */

Hear *PerceptorSet::getHear(int index)
{
	return &hear[index];
}

/**
 * Desruktor ~PerceptorSet() zrusi podtriedy, ktore boli potrebne na rozlozenie prijatej spravy, ktora obsahovala perceptory.
 */

PerceptorSet::~PerceptorSet()
{
	delete[] hear;
	delete[] see;
	delete gs;
	delete time;
	delete[] gyr;
	delete[] frp;
	delete[] hj;
}
