#include "WorldModel.h"
#include <string.h>

/**
 * Konstruktor WorldModel() inicializuje prvky a vytvori podtriedy, ktore budu potrebne na transformaciu dat zo sady perceptorov a spatnu transformaciu dat do sady efektorov.
 * @param team - Tim agenta.
 * @param id - Cislo agenta.
 * @param playGroundLength - Dlzka ihriska.
 * @param playGroundWidth - Sirka ihriska.
 * @param goalWidth - Sirka brany.
 * @param goalHeight - Vyska brany.
 */

WorldModel::WorldModel(char *team, int id, float playGroundLength, float playGroundWidth, float goalWidth, float goalHeight)
{
	playTime = 0;
	playMode[0] = '\0';
	fixedObject = new FixedObject[8]();
	me = new Me(team, id, playGroundLength, playGroundWidth, goalWidth, goalHeight, fixedObject[F1R].getPosition(), fixedObject[G1R].getPosition(), fixedObject[G2R].getPosition(), fixedObject[F2R].getPosition(), fixedObject[F2L].getPosition(), fixedObject[G2L].getPosition(), fixedObject[G1L].getPosition(), fixedObject[F1L].getPosition());
	playerCount = 0;
	player = new Player[21]();
	ballCount = 0;
	ball = new Ball[1]();
}

/**
 * Metoda transform() transformuje data zo sady perceptorov.
 * @param perceptorSet - Ukazovatel na sadu perceptorov, z ktorej sa transformuju data.
 */

void WorldModel::transform(PerceptorSet *perceptorSet)
{
	GS *gs = perceptorSet->getGS();
	playTime = gs->getT()->getTime();
#ifdef WIN32
	strcpy_s(playMode, gs->getPm()->getPlayMode());
#else
	strcpy(playMode, gs->getPm()->getPlayMode());
#endif
	me->transform(perceptorSet);
	if (perceptorSet->getSeeCount() > 0)
	{
		See *see = perceptorSet->getSee(0);
		playerCount = see->getPCount();
		for (int i = 0; i < playerCount; i ++)
		{
			P *p = see->getP(i);
			Pol *pol = p->getPol();
			player[i].transform(p->getTeam()->getTeam(), p->getId()->getId(), me->getPosition(), me->getRotationMatrix(), pol->getDistance(), pol->getAngle1(), pol->getAngle2());
		}
		ballCount = see->getBCount();
		for (int i = 0; i < ballCount; i ++)
		{
			Pol *pol = see->getB(i)->getPol();
			ball[i].transform(me->getPosition(), me->getRotationMatrix(), pol->getDistance(), pol->getAngle1(), pol->getAngle2());
		}
	}
}

/**
 * Metoda reverseTransform() spatne transformuje data do sady efektorov.
 * @param effectorSet - Ukazovatel na sadu efektorov, do ktorej sa spatne transformuju data.
 */

void WorldModel::reverseTransform(EffectorSet *effectorSet)
{
	me->reverseTransform(effectorSet);
}

/**
 * Metoda getPlayTime() vrati hraci cas.
 * @return Hraci cas.
 */

float WorldModel::getPlayTime()
{
	return playTime;
}

/**
 * Metoda getPlayMode() vrati hraci mod.
 * @return Hraci mod.
 */

char *WorldModel::getPlayMode()
{
	return playMode;
}

/**
 * Metoda getFixedObject() vrati ukazovatel na pozadovany pevny objekt.
 * @param index - Cislo pozadovaneho pevneho objektu. Moze nadobudat hodnoty:
 * <ul>
 * 	<li>F1R</li>
 * 	<li>G1R</li>
 * 	<li>G2R</li>
 * 	<li>F2R</li>
 * 	<li>F2L</li>
 * 	<li>G2L</li>
 * 	<li>G1L</li>
 * 	<li>F1L</li>
 * </ul>
 * @return Ukazovatel na pozadovany pevny objekt.
 */

FixedObject *WorldModel::getFixedObject(int index)
{
	return &fixedObject[index];
}

/**
 * Metoda getMe() vrati ukazovatel na mna.
 * @return Ukazovatel na mna.
 */

Me *WorldModel::getMe()
{
	return me;
}

/**
 * Metoda getPlayerCount() vrati pocet videnych hracov.
 * @return Pocet videnych hracov.
 */

int WorldModel::getPlayerCount()
{
	return playerCount;
}

/**
 * Metoda getPlayer() vrati ukazovatel na pozadovaneho videneho hraca.
 * @param index - Cislo pozadovaneho videneho hraca.
 * @return Ukazovatel na pozadovaneho videneho hraca.
 */

Player *WorldModel::getPlayer(int index)
{
	return &player[index];
}

/**
 * Metoda getBallCount() vrati pocet videnych lopt.
 * @return Pocet videnych lopt.
 */

int WorldModel::getBallCount()
{
	return ballCount;
}

/**
 * Metoda getBall() vrati ukazovatel na pozadovanu videnu loptu.
 * @param index - Cislo pozadovanej videnej lopty.
 * @return Ukazovatel na pozadovanu videnu loptu.
 */

Ball *WorldModel::getBall(int index)
{
	return &ball[index];
}

/**
 * Destruktor ~WorldModel() zrusi podtriedy, ktore boli potrebne na transformaciu dat zo sady perceptorov a spatnu transformaciu dat do sady efektorov.
 */

WorldModel::~WorldModel()
{
	delete[] ball;
	delete[] player;
	delete me;
	delete[] fixedObject;
}
